@@ -6,10 +6,13 @@ use {
     std::collections::HashMap,
 };
 
-const TWO15: u32 = 32768;
-const TWO14: u32 = 16384; // 2^14
-                          // const TWO16: u32 = 65536; // 2^16
-const TWO18: u32 = 262144; // 2^18
+#[allow(dead_code)]
+const TWO15: u64 = 32768;
+#[allow(dead_code)]
+const TWO14: u64 = 16384; // 2^14
+const TWO16: u64 = 65536; // 2^16
+#[allow(dead_code)]
+const TWO18: u64 = 262144; // 2^18
 
 /// Type that captures a discrete log challenge.
 ///
@@ -23,65 +26,58 @@ pub struct DiscreteLog {
 }
 
 #[derive(Serialize, Deserialize, Default)]
-pub struct DecodeU32Precomputation(HashMap<[u8; 32], u32>);
+pub struct DecodePrecomputation(HashMap<[u8; 32], u16>);
 
-/// Builds a HashMap of 2^18 elements
-fn decode_u32_precomputation(generator: RistrettoPoint) -> DecodeU32Precomputation {
+/// Builds a HashMap of 2^16 elements
+#[allow(dead_code)]
+fn decode_u32_precomputation(generator: RistrettoPoint) -> DecodePrecomputation {
     let mut hashmap = HashMap::new();
 
-    let two12_scalar = Scalar::from(TWO14);
+    let two16_scalar = Scalar::from(TWO16);
     let identity = RistrettoPoint::identity(); // 0 * G
-    let generator = two12_scalar * generator; // 2^12 * G
+    let generator = two16_scalar * generator; // 2^16 * G
 
     // iterator for 2^12*0G , 2^12*1G, 2^12*2G, ...
     let ristretto_iter = RistrettoIterator::new(identity, generator);
-    let mut steps_for_breakpoint = 0;
-    ristretto_iter.zip(0..TWO18).for_each(|(elem, x_hi)| {
+    ristretto_iter.zip(0..TWO16).for_each(|(elem, x_hi)| {
         let key = elem.compress().to_bytes();
-        hashmap.insert(key, x_hi);
-
-        // unclean way to print status update; will clean up later
-        if x_hi % TWO15 == 0 {
-            println!("     [{:?}/8] completed", steps_for_breakpoint);
-            steps_for_breakpoint += 1;
-        }
+        hashmap.insert(key, x_hi as u16);
     });
-    println!("     [8/8] completed");
 
-    DecodeU32Precomputation(hashmap)
+    DecodePrecomputation(hashmap)
 }
 
 lazy_static::lazy_static! {
     /// Pre-computed HashMap needed for decryption. The HashMap is independent of (works for) any key.
-    pub static ref DECODE_U32_PRECOMPUTATION_FOR_G: DecodeU32Precomputation = {
-        static DECODE_U32_PRECOMPUTATION_FOR_G_BINCODE: &[u8] =
+    pub static ref DECODE_PRECOMPUTATION_FOR_G: DecodePrecomputation = {
+        static DECODE_PRECOMPUTATION_FOR_G_BINCODE: &[u8] =
             include_bytes!("decode_u32_precomputation_for_G.bincode");
-        bincode::deserialize(DECODE_U32_PRECOMPUTATION_FOR_G_BINCODE).unwrap_or_default()
+        bincode::deserialize(DECODE_PRECOMPUTATION_FOR_G_BINCODE).unwrap_or_default()
     };
 }
 
-/// Solves the discrete log instance using a 18/14 bit offline/online split
+/// Solves the discrete log instance using a 16/16 bit offline/online split
 impl DiscreteLog {
     /// Solves the discrete log problem under the assumption that the solution
     /// is a 32-bit number.
-    pub(crate) fn decode_u32(self) -> Option<u32> {
-        self.decode_u32_online(&decode_u32_precomputation(self.generator))
+    pub(crate) fn decode_u32(self) -> Option<u64> {
+        self.decode_online(&DECODE_PRECOMPUTATION_FOR_G, TWO16)
     }
 
-    /// Solves the discrete log instance using the pre-computed HashMap by enumerating through 2^14
-    /// possible solutions
-    pub fn decode_u32_online(self, hashmap: &DecodeU32Precomputation) -> Option<u32> {
+    pub fn decode_online(self, hashmap: &DecodePrecomputation, solution_bound: u64) -> Option<u64> {
         // iterator for 0G, -1G, -2G, ...
         let ristretto_iter = RistrettoIterator::new(self.target, -self.generator);
 
         let mut decoded = None;
-        ristretto_iter.zip(0..TWO14).for_each(|(elem, x_lo)| {
-            let key = elem.compress().to_bytes();
-            if hashmap.0.contains_key(&key) {
-                let x_hi = hashmap.0[&key];
-                decoded = Some(x_lo + TWO14 * x_hi);
-            }
-        });
+        ristretto_iter
+            .zip(0..solution_bound)
+            .for_each(|(elem, x_lo)| {
+                let key = elem.compress().to_bytes();
+                if hashmap.0.contains_key(&key) {
+                    let x_hi = hashmap.0[&key];
+                    decoded = Some(x_lo + solution_bound * x_hi as u64);
+                }
+            });
         decoded
     }
 }
@@ -122,7 +118,7 @@ mod tests {
     fn test_serialize_decode_u32_precomputation_for_G() {
         let decode_u32_precomputation_for_G = decode_u32_precomputation(G);
 
-        if decode_u32_precomputation_for_G.0 != DECODE_U32_PRECOMPUTATION_FOR_G.0 {
+        if decode_u32_precomputation_for_G.0 != DECODE_PRECOMPUTATION_FOR_G.0 {
             use std::{fs::File, io::Write, path::PathBuf};
             let mut f = File::create(PathBuf::from(
                 "src/encryption/decode_u32_precomputation_for_G.bincode",
@@ -134,14 +130,9 @@ mod tests {
         }
     }
 
-    /// Discrete log test for 16/16 split
-    ///
-    /// Very informal measurements on my machine:
-    ///   - 8 sec for precomputation
-    ///   - 3 sec for online computation
     #[test]
     fn test_decode_correctness() {
-        let amount: u32 = 65545;
+        let amount: u64 = 65545;
 
         let instance = DiscreteLog {
             generator: G,
@@ -154,7 +145,7 @@ mod tests {
         let precomputation_secs = start_precomputation.elapsed().as_secs_f64();
 
         let start_online = Instant::now();
-        let computed_amount = instance.decode_u32_online(&precomputed_hashmap).unwrap();
+        let computed_amount = instance.decode_online(&precomputed_hashmap, TWO16).unwrap();
         let online_secs = start_online.elapsed().as_secs_f64();
 
         assert_eq!(amount, computed_amount);
